function MovingTargetBenchmark(parameters, model, demos, opt_sim) %function running the moving target benchmark.

disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
disp('%%%%%%%%%% Running Moving Target Benchmark %%%%%%%%%%');
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');

% find initial points of all demonstrations
d = size(demos{1}.pos,1);
x0_all = [];
for i=1:length(demos)
    x0_all(:,i) = demos{i}.pos(:,1);
end

numSimulations = length(parameters.amplitude);

disp(['Running ', num2str(numSimulations), ' simulations.']);
results = [];

iteration = 1;
waitBarHandler = waitbar(0,'Moving Target Benchmark: 0 % completed...');

% prepare PL global analysis of demos
highPLR2=parameters.demoEvaulation.alldemos_high;
R2Demos=parameters.demoEvaulation.totalR2_demo(highPLR2,1);

motionDuration = zeros(1,length(demos));
for i=1:length(demos)
    motionDuration(i) = demos{i}.t(end);
end
motionDuration = mean(motionDuration);

if (length(parameters.start_time) ~= length(parameters.amplitude)) || ...
        (length(parameters.start_time) ~= size(parameters.vector,2)) || ...
        (length(parameters.start_time) ~= length(parameters.duration))
    close(waitBarHandler);
    errordlg('Wrong definition of benchmark parameters! Check if they all have the same size.','Benchmark aborted!','modal');
    model.doAbourt();
    return
end

for iter = 1:length(parameters.start_time)
    start_time = parameters.start_time(iter)*motionDuration;
    duration = parameters.duration(iter)*motionDuration;
    amplitude = parameters.amplitude(iter);
    direction = parameters.vector(:,iter);
    
    if model.checkAbourt
        break;
    end
    vector = direction / norm(direction) * amplitude;
    
    disp(['target move start time [s]: ' num2str(start_time) ' - target move duration [s]: ' num2str(duration) ...
        ' amplitude: ' num2str(amplitude) ' - direction: (' num2str(direction(1)) ', ' num2str(direction(2)) ')']);
    
    opt_sim.perturbation.type = 'tcp'; %moving target continuously
    opt_sim.perturbation.t0 = start_time;
    opt_sim.perturbation.tf = start_time + duration;
    opt_sim.perturbation.dx = vector; %we might as well randomize the perturbations
    opt_sim.plot = 0; %parameters.plot;
    
    %running the simulator with the initial points of all demonstrations
    [x xd t xT computationTime] = Simulation(x0_all,[0; 0],model,opt_sim);
    if model.checkAbourt
        break;
    end
    %conditions of the test
    param_values(iteration).start_time = start_time;
    param_values(iteration).duration = duration;
    param_values(iteration).amplitude = amplitude;
    param_values(iteration).direction = direction;
    
    repros_p=cell(size(demos));
    matched_repros=cell(size(demos));
    
    
    [tmpVaris, R2iw]=AnalyzeDemosPowerLaw(x,[],repmat(opt_sim.dt,size(x,2),1),parameters.demoEvaulation.R2threshold,parameters.demoEvaulation.TimeWindows,0); % send data, draw==0
    % calculate the percentage change between the R2 of the
    % demonstration and that of the reproduction. 50% means that the R2 of the reproduction was 1.5 times that of the demonstration
    
    PL_ReproductionVsDemos=...
        mean((R2iw(:,highPLR2,1)-repmat(R2Demos',size(x,2),1))./...
        (repmat(R2Demos',size(x,2),1))*100,2);
    
    for i=1:length(demos)
        %% Seperate Kinematics from Geometric
        
        pos= x{i}';
        vel= xd{i}';
        jerk = diff(xd{i}, 2, 2) / opt_sim.dt^2; %motion's jerk
        max_jerk = max(sum(jerk.^2,1)).^.5; % max jerk of all demos
        % avoid integrating the 2% of possible command related jerk spikes
        sq_norm_jerk=sum(jerk.^2,1);
        thresh_cont_jerk=prctile(sq_norm_jerk,98);
        mean_jerk = mean(sum(jerk(:,sq_norm_jerk<=thresh_cont_jerk).^2,1)).^.5;  % mean jerk of all demos
        
        e= find((sum(vel.^2,2).^.5)>0.0,1,'first');
        s= find((sum(vel.^2,2).^.5)>0.0,1,'last');
        pos = pos(e:s,:);
        vel = vel(e:s,:);
        if size(pos,1)<3
            warning('Model did not perform any trajectory. Benchmark will be rejected.')
            model.doAbourt();
            break;
        else
            % main eval
            [results_tmp, log_tmp]= generalEvaluation(demos{i}.pos', pos, xT');
            
            % plotting informations
            repros_p{i} = [log_tmp.pos_r';gradient(log_tmp.pos_r')/demos{i}.dt];
            matched_repros{i} = log_tmp.matched_r';
            
            % additional evaluation
            results(iteration).demos(i)= results_tmp;
            results(iteration).demos(i).computationTime = computationTime{i}; % check this line out
            results(iteration).demos(i).normalizedFinalTime = t(s)/demos{i}.t(end);
            
            results(iteration).demos(i).max_jerk = max_jerk;
            results(iteration).demos(i).mean_jerk = mean_jerk;
            results(iteration).demos(i).PL_R2 = PL_ReproductionVsDemos(i);
            results(iteration).demos(i).target_error.position = norm(x{i}(:,end) - xT(:,end));
            results(iteration).demos(i).target_error.velocity = norm(xd{i}(:,end));
            
            %% save some additional information
            log(iteration).demos(i).trajectory = x{i};
            log(iteration).demos(i).velocity = xd{i};
            log(iteration).demos(i).jerk = jerk;
            log(iteration).demos(i).err = log_tmp.err;
        end
        if model.checkAbourt
            break;
        end
    end
    if model.checkAbourt
        break;
    end
    results(iteration).modelSize = model.bytes;
    
    percentage = round(iteration * 100 / numSimulations);
    disp([' ============', num2str(percentage), '% completed ============.']);
    waitbar(iteration / numSimulations, waitBarHandler, ['Moving Target Benchmark: ' num2str(percentage) ' % completed...']);
    
    if(parameters.plot)
        options.animation=parameters.animation;
        options.matched_repros=matched_repros;
        if isfield(parameters, 'parentup')
            options.parentup =  parameters.parentup;
        end
        PlotResults(repros_p, demos, xT, results, options);
        if(parameters.plot_wait_time == -1)
            disp('Press ENTER to run next simulation.');
            pause;
        else
            pause(abs(parameters.plot_wait_time));
        end
    end
    iteration = iteration + 1;
end


close(waitBarHandler);
if model.checkAbourt
    disp('Benchmark aborted!');
else
    disp('Saving results ...');
    save(parameters.resultFileWithPath, 'param_values', 'results', 'log');
    disp(['Results saved to ', parameters.resultFileWithPath]);
end
end