function [x xd t xT computationTime]=Simulation(x0,xT,model,varargin)
%
% This function simulates motions that are in the form of:
%
%                               xd=f(x)
%
% where x is an arbitrary d dimensional variable, and xd is its first time
% derivative.
%
% The function can be called using:
%       [x xd t]=Simulation(x0,xT,model)
%
% or
%       [x xd t]=Simulation(x0,xT,model,options)
%
% to also pass a structure of desired options.
%
% Please look at the two parts of the simulator marked by:
%                   < should be set by the user >
% in order to integrate your model into the simulator.
%
% Inputs -----------------------------------------------------------------
%   o x:       d x N Column vector representing N different starting point(s)
%   o xT:      d x 1 Column vector representing the target point
%   o model:   The model of the motion
%
%   o options: A structure to set the optional parameters of the simulator.
%              The following parameters can be set in the options:
%       - .dt:      integration time step [default: 0.02]
%       - .i_max:   maximum number of iteration for simulator [default: i_max=1000]
%       - .plot     setting simulation graphic on (true) or off (false) [default: true]
%       - .tol:     A positive scalar defining the threshold to stop the
%                   simulator. If the motions velocity becomes less than
%                   tol, then simulation stops [default: 0.001]
%       - .perturbation: a structure to apply pertorbations to the robot.
%                        This variable has the following subvariables:
%       - .perturbation.type: A string defining the type of perturbations.
%                             The acceptable values are:
%                             'tdp' : target discrete perturbation
%                             'tcp' : target continuous perturbation
%                             'rdp' : robot discrete perturbation
%                             'rcp' : robot continuous perturbation
%       - .perturbation.t0:   A positive scalar defining the time when the
%                             perturbation should be applied.
%       - .perturbation.tf:   A positive scalar defining the final time for
%                             the perturbations. This variable is necessary
%                             only when the type is set to 'tcp' or 'rcp'.
%       - .perturbation.dx:   A d x 1 vector defining the perturbation's
%                             magnitude. In 'tdp' and 'rdp', it simply
%                             means a relative displacement of the
%                             target/robot with the vector dx. In 'tcp' and
%                             'rcp', the target/robot starts moving with
%                             the velocity dx.
%
% Outputs ----------------------------------------------------------------
%   o x:       d x T x N matrix containing the position of N, d dimensional
%              trajectories with the length T.
%
%   o xd:      d x T x N matrix containing the velocity of N, d dimensional
%              trajectories with the length T.
%
%   o t:       1 x N vector containing the trajectories' time.
%
%   o xT:      A matrix recording the change in the target position. Useful
%              only when 'tcp' or 'tdp' perturbations applied.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%    Copyright (c) 2010 S. Mohammad Khansari-Zadeh, LASA Lab, EPFL,   %%%
%%%          CH-1015 Lausanne, Switzerland, http://lasa.epfl.ch         %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% The program is free for non-commercial academic use. Please contact the
% author if you are interested in using the software for commercial purposes.
% The software must not be modified or distributed without prior permission
% of the authors. Please acknowledge the authors in any academic publications
% that have made use of this code or part of it. Please use this BibTex
% reference:
%
%  S. M. Khansari-Zadeh and A. Billard, "Learning Stable Non-Linear Dynamical
%  Systems with Gaussian Mixture Models", IEEE Transaction on Robotics, 2011.
%
% To get latest upadate of the software please visit
%                          http://lasa.epfl.ch/khansari
%
% Please send your feedbacks or questions to:
%                           mohammad.khansari_at_epfl.ch


%% parsing inputs
if isempty(varargin)
    options = check_options();
else
    options = check_options(varargin{1}); % Checking the given options, and add ones are not defined.
end

d=size(x0,1); %dimension of the model
if isempty(xT)
    xT = zeros(d,1);
end

if d~=length(xT)
    disp('Error: length(x0) should be equal to length(xT)!')
    x=[];xd=[];t=[];
    return
end
%% setting initial values

nbSPoint=size(x0,2); %number of starting points. This enables to simulatneously run several trajectories
modelContainter = cell(nbSPoint,1);
%initialization
for i=1:nbSPoint
    modelContainter{i} = model.copy();
    x{i}(:,1) = x0(:,i);
    xd{i}(:,1) = zeros(d,1);
    modelContainter{i}.setDt(options.dt);
end

warning('off','MATLAB:nearlySingularMatrix'); %postpone plotting potential subclass warnings

t=0; %starting time

%% Simulation
i=1;
stopping = false;
while ~stopping
    %Computing xd
    stopping = model.checkAbourt();
    tmpStopping = zeros(nbSPoint,1);
    for k=1:nbSPoint
        tmpStopping(k) = modelContainter{k}.checkStopped();
        
        if ~tmpStopping(k)
            modelContainter{k}.setCurrentTimeStep(i);
            modelContainter{k}.setDt(options.dt);
            modelContainter{k}.setCurrentPos(x{k}(:,i));
            if i> 1
                modelContainter{k}.setCurrentVel(xd{k}(:,i-1));
            else
                modelContainter{k}.setCurrentVel(xd{k}(:,i));
            end
            
            modelContainter{k}.setCurrentTargetPos(xT(:,end));
            if t <= options.obstacle.appear_time %the case where the obstacle does not exist
                
                modelContainter{k}.setObstacleDetected(0);
            else %the case where the obstacle appears
                modelContainter{k}.setObstacleDetected(1);
                modelContainter{k}.setObstacle(options.obstacle)
                
            end
            
            tic
            xd{k}(:,i) = modelContainter{k}.step();
            computationTime{k}(i) = toc;
            
            tmpStopping(k) = modelContainter{k}.checkStopped();
            
            if ~tmpStopping(k)
                x{k}(:,i+1) = x{k}(:,i) + xd{k}(:,i)*options.dt;
                
                
                %applying perturbation if any
                switch options.perturbation.type
                    case 'rdp' %applying robot discrete perturbation
                        if i == round(options.perturbation.t0/options.dt)+1 && length(options.perturbation.dx)==d
                            x{k}(:,i+1) = x{k}(:,i+1) + options.perturbation.dx;
                        end
                    case 'rcp' %applying robot continuous perturbation
                        if i >= round(options.perturbation.t0/options.dt)+1 && i <= round(options.perturbation.tf/options.dt) && length(options.perturbation.dx)==d
                            x{k}(:,i+1) = x{k}(:,i+1) + options.perturbation.dx*options.dt;
                        end
                end
            end
        end
    end
    
    allModelsFinished_b = all(tmpStopping);
    if ~allModelsFinished_b
        t(i+1)=t(i)+options.dt;
    end
    
    %applying target perturbation if any
    switch options.perturbation.type
        case 'tdp' %applying target discrete perturbation
            if i == round(options.perturbation.t0/options.dt)+1 && length(options.perturbation.dx)==d
                xT(:, end+1) = xT(:,end) + options.perturbation.dx;
            else
                xT(:, end+1) = xT(:, end);
            end
        case 'tcp' %applying target continuous perturbation
            if i >= round(options.perturbation.t0/options.dt)+1 && i <= round(options.perturbation.tf/options.dt) && length(options.perturbation.dx)==d
                xT(:, end+1) = xT(:, end) + options.perturbation.dx * options.dt;
            else
                xT(:, end+1) = xT(:, end);
            end
        otherwise
            xT(:, end+1) = xT(:, end);
    end
    
    
    %If all methods signaled stop or if i > options.i_max , then the simulation stops.
    if i > 3 && allModelsFinished_b || i>options.i_max-2
        disp(['All models have finished: ' num2str(allModelsFinished_b)])
        
        if i>options.i_max-2
            fprintf('Simulation stopped since it reaches the maximum number of allowed iterations i_max = %1.0f\n',i)
            fprintf('Exiting without convergence!!! Increase the parameter ''options.i_max'' to handle this error.\n')
        end
        break
    end
    i=i+1;
end
warning('on','MATLAB:nearlySingularMatrix'); % resume plotting subclass warnings

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function options = check_options(varargin)
if ~isempty(varargin)
    options = varargin{1};
else
    options.dt=0.02; %to create the variable
end

if ~isfield(options,'dt') % integration time step
    options.dt = 0.02;
end
if ~isfield(options,'i_max') % maximum number of iterations
    options.i_max = 1000;
end
% if ~isfield(options,'tol') % convergence tolerance
%     options.tol = 0.001;
% end
if ~isfield(options,'plot') % shall simulator plot the figure
    options.plot = 1;
else
    options.plot = options.plot > 0;
end

if ~isfield(options,'perturbation') % shall simulator plot the figure
    options.perturbation.type = '';
elseif ~isfield(options.perturbation,'type') || ~isfield(options.perturbation,'t0') || ~isfield(options.perturbation,'dx') || ...
        ((strcmpi(options.perturbation.type,'rcp') || strcmpi(options.perturbation.type,'tcp')) && ~isfield(options.perturbation,'tf')) || ...
        (~strcmpi(options.perturbation.type,'rcp') && ~strcmpi(options.perturbation.type,'tcp') && ~strcmpi(options.perturbation.type,'rdp') && ~strcmpi(options.perturbation.type,'tdp'))
    
    disp('Invalid perturbation structure. The perturbation input is ignored!')
    options.perturbation.type = '';
end

if ~isfield(options,'obstacle') % shall simulator plot the figure
    options.obstacle.appear_time = inf;
elseif ~isfield(options.obstacle,'appear_time') || ~isfield(options.obstacle,'position') || ~isfield(options.obstacle,'radius')
    disp('Invalid obstacle structure. The obstacle input is ignored!')
    options.obstacle.appear_time = inf;
end
end