% Function to compute the inverse kinematics of the iCub
% q_elb: 
% target_pos: desired target position (in mm, 3D, in the root reference frame
%               of the robot)
% q: resulting joint angles for the shoulder and the elbow (4D)
% err: error (in mm)
% varagin: one optional parameter to specify the initial position of the
%           arm of the robot (4D)
function [q, err] = InverseKinematics(target_pos, varargin) 

q_init = zeros(1,4); %resulting joint angles
[LB UB] = JointLimits();

%% Checking parameter consistency
if (length(target_pos) ~= 3)
    disp('Wrong target position dimension. Please provide 3D points in the root reference frame of the robot.');
    q = [0 0 0 0];
    err = Inf;
    return;
end
if (~isempty(varargin) && length(q_init) ~= 4)
    disp('Wrong initial joint angles dimension. Please provide joints angle for the four first DOF.');
    q = [0 0 0 0];
    err = Inf;
    return;
end


%% Initial conditions and options
options = optimset('GradObj', 'on', 'Display', 'off');
    %options for the optimization :
    % GradObj: user defined gradient function, 
    % Display: 'off' displays no output.
    %         'iter' displays output at each iteration, and gives the default exit message.
    %         'iter-detailed' displays output at each iteration, and gives the technical exit message.
    %         'notify' displays output only if the function does not converge, and gives the default exit message.
    %         'notify-detailed' displays output only if the function does not converge, and gives the technical exit message.
    %         'final' displays just the final output, and gives the default exit message.
    %         'final-detailed' displays just the final output, and gives the technical exit message.

    
%% Main Computation
    q = fmincon(@(q)(fun(q, target_pos)), q_init, [], [], [], [], LB, UB, [], options); % non linear solver
    err = sqrt(fun(q, target_pos)); %the error on the end-effector, in mm
    
end



%% Function to minimize
function [f, g] = fun(q, target_pos)

T0 = ArmTRo0();
DH = DHParams();


DHMat1 = EvalDHMatrix(q(1), DH{4});
DHMat2 = EvalDHMatrix(q(2), DH{5});
DHMat3 = EvalDHMatrix(q(3), DH{6});
DHMat4 = EvalDHMatrix(q(4), DH{7});

dDHMat1 = EvalDHDerivMatrix(q(1), DH{4});
dDHMat2 = EvalDHDerivMatrix(q(2), DH{5});
dDHMat3 = EvalDHDerivMatrix(q(3), DH{6});
dDHMat4 = EvalDHDerivMatrix(q(4), DH{7});

notActuated = EvalDHMatrix(0, DH{8}) * EvalDHMatrix(0, DH{9}) * EvalDHMatrix(0, DH{10}) * [0 0 0 1]'; % the last 3 joints (not actuated)
T = T0 * DHMat1 * DHMat2 * DHMat3 * DHMat4 * notActuated; % Computing the end effector position.
v = T(1:3) - target_pos'; % distance vector between the desired position and the end effector position.
f = norm(v)^2; % the function to minimize is the squared error between the desired and achieved positions.

v =  2 * v; 

if nargout>1
    dTdq1 = T0 * dDHMat1 * DHMat2 * DHMat3 * DHMat4 * notActuated;
    dTdq2 = T0 * DHMat1 * dDHMat2 * DHMat3 * DHMat4 * notActuated;
    dTdq3 = T0 * DHMat1 * DHMat2 * dDHMat3 * DHMat4 * notActuated;
    dTdq4 = T0 * DHMat1 * DHMat2 * DHMat3 * dDHMat4 * notActuated;
    
    %the gradient to provide to fmincon
    g = [ dot(v, dTdq1(1:3));
          dot(v, dTdq2(1:3));
          dot(v, dTdq3(1:3));
          dot(v, dTdq4(1:3))];
end
end


